Stan Math Library: stan/math/prim/scal/prob/frechet_log.hpp Source File

| Stan Math Library 2.10.0 reverse mode automatic differentiation

- Main Page
- Modules
- Namespaces
- Classes
- Files

- File List
- File Members

frechet_log.hpp
Go to the documentation of this file.
1 #ifndef STAN_MATH_PRIM_SCAL_PROB_FRECHET_LOG_HPP
2 #define STAN_MATH_PRIM_SCAL_PROB_FRECHET_LOG_HPP
3
4 #include <boost/random/weibull_distribution.hpp>
5 #include <boost/random/variate_generator.hpp>
6 #include <stan/math/prim/scal/meta/OperandsAndPartials.hpp>
7 #include <stan/math/prim/scal/err/check_consistent_sizes.hpp>
8 #include <stan/math/prim/scal/err/check_nonnegative.hpp>
9 #include <stan/math/prim/scal/err/check_not_nan.hpp>
10 #include <stan/math/prim/scal/err/check_positive.hpp>
11 #include <stan/math/prim/scal/err/check_positive_finite.hpp>
12 #include <stan/math/prim/scal/fun/log1m.hpp>
13 #include <stan/math/prim/scal/fun/multiply_log.hpp>
14 #include <stan/math/prim/scal/fun/value_of.hpp>
15 #include <stan/math/prim/scal/meta/length.hpp>
16 #include <stan/math/prim/scal/meta/is_constant_struct.hpp>
17 #include <stan/math/prim/scal/meta/VectorView.hpp>
18 #include <stan/math/prim/scal/meta/VectorBuilder.hpp>
19 #include <stan/math/prim/scal/meta/partials_return_type.hpp>
20 #include <stan/math/prim/scal/meta/return_type.hpp>
21 #include <stan/math/prim/scal/fun/constants.hpp>
22 #include <stan/math/prim/scal/meta/include_summand.hpp>
23 #include <cmath>
24
25 namespace stan {
26
27 namespace math {
28
29 // Frechet(y|alpha, sigma) [y > 0; alpha > 0; sigma > 0]
30 // FIXME: document
31 template <bool propto,
32 typename T_y, typename T_shape, typename T_scale>
33 typename return_type<T_y, T_shape, T_scale>::type
34 frechet_log(const T_y& y, const T_shape& alpha, const T_scale& sigma) {
35 static const char* function("stan::math::frechet_log");
36 typedef typename stan::partials_return_type<T_y, T_shape, T_scale>::type
37 T_partials_return;
38
39 using stan::math::check_positive;
40 using stan::math::check_not_nan;
41 using stan::math::check_positive_finite;
42 using stan::math::value_of;
43 using stan::math::check_consistent_sizes;
44 using stan::math::multiply_log;
45 using std::log;
46
47 // check if any vectors are zero length
48 if (!(stan::length(y)
49 && stan::length(alpha)
50 && stan::length(sigma)))
51 return 0.0;
52
53 // set up return value accumulator
54 T_partials_return logp(0.0);
55 check_positive(function, "Random variable", y);
56 check_positive_finite(function, "Shape parameter", alpha);
57 check_positive_finite(function, "Scale parameter", sigma);
58 check_consistent_sizes(function,
59 "Random variable", y,
60 "Shape parameter", alpha,
61 "Scale parameter", sigma);
62
63 // check if no variables are involved and prop-to
64 if (!include_summand<propto, T_y, T_shape, T_scale>::value)
65 return 0.0;
66
67 VectorView<const T_y> y_vec(y);
68 VectorView<const T_shape> alpha_vec(alpha);
69 VectorView<const T_scale> sigma_vec(sigma);
70 size_t N = max_size(y, alpha, sigma);
71
72 VectorBuilder<include_summand<propto, T_shape>::value,
73 T_partials_return, T_shape> log_alpha(length(alpha));
74 for (size_t i = 0; i < length(alpha); i++)
75 if (include_summand<propto, T_shape>::value)
76 log_alpha[i] = log(value_of(alpha_vec[i]));
77
78 VectorBuilder<include_summand<propto, T_y, T_shape>::value,
79 T_partials_return, T_y> log_y(length(y));
80 for (size_t i = 0; i < length(y); i++)
81 if (include_summand<propto, T_y, T_shape>::value)
82 log_y[i] = log(value_of(y_vec[i]));
83
84 VectorBuilder<include_summand<propto, T_shape, T_scale>::value,
85 T_partials_return, T_scale> log_sigma(length(sigma));
86 for (size_t i = 0; i < length(sigma); i++)
87 if (include_summand<propto, T_shape, T_scale>::value)
88 log_sigma[i] = log(value_of(sigma_vec[i]));
89
90 VectorBuilder<include_summand<propto, T_y, T_shape, T_scale>::value,
91 T_partials_return, T_y> inv_y(length(y));
92 for (size_t i = 0; i < length(y); i++)
93 if (include_summand<propto, T_y, T_shape, T_scale>::value)
94 inv_y[i] = 1.0 / value_of(y_vec[i]);
95
96 VectorBuilder<include_summand<propto, T_y, T_shape, T_scale>::value,
97 T_partials_return, T_y, T_shape, T_scale>
98 sigma_div_y_pow_alpha(N);
99 for (size_t i = 0; i < N; i++)
100 if (include_summand<propto, T_y, T_shape, T_scale>::value) {
101 const T_partials_return alpha_dbl = value_of(alpha_vec[i]);
102 sigma_div_y_pow_alpha[i] = pow(inv_y[i] * value_of(sigma_vec[i]),
103 alpha_dbl);
104 }
105
106 OperandsAndPartials<T_y, T_shape, T_scale>
107 operands_and_partials(y, alpha, sigma);
108 for (size_t n = 0; n < N; n++) {
109 const T_partials_return alpha_dbl = value_of(alpha_vec[n]);
110 if (include_summand<propto, T_shape>::value)
111 logp += log_alpha[n];
112 if (include_summand<propto, T_y, T_shape>::value)
113 logp -= (alpha_dbl+1.0)*log_y[n];
114 if (include_summand<propto, T_shape, T_scale>::value)
115 logp += alpha_dbl*log_sigma[n];
116 if (include_summand<propto, T_y, T_shape, T_scale>::value)
117 logp -= sigma_div_y_pow_alpha[n];
118
119 if (!is_constant_struct<T_y>::value) {
120 const T_partials_return inv_y_dbl = value_of(inv_y[n]);
121 operands_and_partials.d_x1[n]
122 += -(alpha_dbl+1.0) * inv_y_dbl
123 + alpha_dbl * sigma_div_y_pow_alpha[n] * inv_y_dbl;
124 }
125 if (!is_constant_struct<T_shape>::value)
126 operands_and_partials.d_x2[n]
127 += 1.0/alpha_dbl
128 + (1.0 - sigma_div_y_pow_alpha[n]) * (log_sigma[n] - log_y[n]);
129 if (!is_constant_struct<T_scale>::value)
130 operands_and_partials.d_x3[n]
131 += alpha_dbl / value_of(sigma_vec[n])
132 * (1 - sigma_div_y_pow_alpha[n]);
133 }
134 return operands_and_partials.value(logp);
135 }
136
137 template <typename T_y, typename T_shape, typename T_scale>
138 inline
139 typename return_type<T_y, T_shape, T_scale>::type
140 frechet_log(const T_y& y, const T_shape& alpha, const T_scale& sigma) {
141 return frechet_log<false>(y, alpha, sigma);
142 }
143 }
144 }
145 #endif
check_not_nan.hpp
stan::math::OperandsAndPartials::d_x2
VectorView< T_return_type, false, true > d_x2
Definition: OperandsAndPartials.hpp:40
stan::math::check_not_nan
bool check_not_nan(const char *function, const char *name, const T_y &y)
Return true if y is not NaN.
Definition: check_not_nan.hpp:63
check_positive_finite.hpp
check_positive.hpp
stan
Definition: log_sum_exp.hpp:8
stan::math::value_of
T value_of(const fvar< T > &v)
Return the value of the specified variable.
Definition: value_of.hpp:16
stan::math::log
fvar< T > log(const fvar< T > &x)
Definition: log.hpp:15
stan::math::OperandsAndPartials::value
T_return_type value(double value)
Returns a T_return_type with the value specified with the partial derivatves.
Definition: OperandsAndPartials.hpp:68
stan::length
size_t length(const std::vector< T > &x)
Definition: length.hpp:10
is_constant_struct.hpp
return_type.hpp
stan::math::include_summand
Template metaprogram to calculate whether a summand needs to be included in a proportional (log) prob...
Definition: include_summand.hpp:36
stan::return_type::type
boost::math::tools::promote_args< typename scalar_type< T1 >::type, typename scalar_type< T2 >::type, typename scalar_type< T3 >::type, typename scalar_type< T4 >::type, typename scalar_type< T5 >::type, typename scalar_type< T6 >::type >::type type
Definition: return_type.hpp:27
include_summand.hpp
multiply_log.hpp
check_nonnegative.hpp
stan::is_constant_struct
Metaprogram to determine if a type has a base scalar type that can be assigned to type double...
Definition: is_constant_struct.hpp:13
stan::math::frechet_log
return_type< T_y, T_shape, T_scale >::type frechet_log(const T_y &y, const T_shape &alpha, const T_scale &sigma)
Definition: frechet_log.hpp:34
stan::math::OperandsAndPartials
This class builds partial derivatives with respect to a set of operands.
Definition: OperandsAndPartials.hpp:38
VectorView.hpp
stan::math::OperandsAndPartials::d_x3
VectorView< T_return_type, false, true > d_x3
Definition: OperandsAndPartials.hpp:41
stan::math::check_positive
bool check_positive(const char *function, const char *name, const T_y &y)
Return true if y is positive.
Definition: check_positive.hpp:68
stan::max_size
size_t max_size(const T1 &x1, const T2 &x2)
Definition: max_size.hpp:9
VectorBuilder.hpp
log1m.hpp
partials_return_type.hpp
OperandsAndPartials.hpp
length.hpp
stan::math::multiply_log
fvar< T > multiply_log(const fvar< T > &x1, const fvar< T > &x2)
Definition: multiply_log.hpp:15
stan::VectorBuilder
VectorBuilder allocates type T1 values to be used as intermediate values.
Definition: VectorBuilder.hpp:28
constants.hpp
stan::math::check_consistent_sizes
bool check_consistent_sizes(const char *function, const char *name1, const T1 &x1, const char *name2, const T2 &x2)
Return true if the dimension of x1 is consistent with x2.
Definition: check_consistent_sizes.hpp:31
stan::math::pow
fvar< T > pow(const fvar< T > &x1, const fvar< T > &x2)
Definition: pow.hpp:18
value_of.hpp
stan::VectorView
VectorView is a template expression that is constructed with a container or scalar, which it then allows to be used as an array using operator[].
Definition: VectorView.hpp:48
stan::partials_return_type::type
boost::math::tools::promote_args< typename partials_type< typename scalar_type< T1 >::type >::type, typename partials_type< typename scalar_type< T2 >::type >::type, typename partials_type< typename scalar_type< T3 >::type >::type, typename partials_type< typename scalar_type< T4 >::type >::type, typename partials_type< typename scalar_type< T5 >::type >::type, typename partials_type< typename scalar_type< T6 >::type >::type >::type type
Definition: partials_return_type.hpp:26
check_consistent_sizes.hpp
stan::math::check_positive_finite
bool check_positive_finite(const char *function, const char *name, const T_y &y)
Return true if y is positive and finite.
Definition: check_positive_finite.hpp:28
stan::math::OperandsAndPartials::d_x1
VectorView< T_return_type, false, true > d_x1
Definition: OperandsAndPartials.hpp:39

[ Stan Home Page ] © 2011–2016, Stan Development Team.